@@ -23,11 +23,6 @@ sub get_http_headers {
 
 sub parse_http_response {
     my ( $self, $resp_obj, $resp_body ) = @_;
-no warnings 'once';
-if ($cPanel::APIClient::DEBUG) {
-use Data::Dumper;
-print STDERR Dumper( $resp_obj );
-}
 
     if ( $resp_obj->code() !~ m<\A2> ) {
         die cPanel::APIClient::X->create( 'HTTP', $resp_obj->as_string() . $/ . $resp_body );
@@ -35,7 +30,9 @@ print STDERR Dumper( $resp_obj );
 
     my $resp_struct = cPanel::APIClient::Utils::JSON::decode($resp_body);
 
-    return $self->HTTP_RESPONSE_CLASS()->new($resp_struct);
+    return $self->HTTP_RESPONSE_CLASS()->new(
+        $self->_EXTRACT_RESPONSE($resp_struct),
+    );
 }
 
 sub create_transport_error {
@@ -44,4 +41,8 @@ sub create_transport_error {
     die cPanel::APIClient::X->create( 'SubTransport', $description );
 }
 
+sub _EXTRACT_RESPONSE {
+    return $_[1];
+}
+
 1;
