@@ -131,11 +131,12 @@ sub _create_easy {
 
     if ( $self->_needs_session() ) {
         $easy->setopt( Net::Curl::Easy::CURLOPT_COOKIEFILE, q<> );
-        $easy->setopt( Net::Curl::Easy::CURLOPT_SHARE(),    $self->{'share'} );
+        $easy->setopt( Net::Curl::Easy::CURLOPT_SHARE,      $self->{'share'} );
     }
 
     # We POST even if there’s no payload.
     $payload = q<> if !defined $payload;
+    $easy->setopt( Net::Curl::Easy::CURLOPT_POSTFIELDSIZE,  length $payload );
     $easy->setopt( Net::Curl::Easy::CURLOPT_COPYPOSTFIELDS, $payload );
 
     $easy->setopt( Net::Curl::Easy::CURLOPT_URL, $url );
@@ -198,7 +199,6 @@ sub request {
         }
 
         my $promise1 = $self->{'promiser'}->add_handle($easy);
-        print STDERR "promise1: $promise1\n";
 
         my $promise2 = $promise1->then(
             sub {
@@ -211,12 +211,10 @@ sub request {
                 die cPanel::APIClient::X->create( 'SubTransport', $str, code => $code );
             },
         );
-        print STDERR "promise2: $promise2\n";
 
         my $promise3 = $promise2->finally( sub {
             $settled = 1;
         } );
-        print STDERR "promise3: $promise3\n";
 
         return $promise3;
     };
