@@ -12,6 +12,7 @@ use warnings;
 use autodie;
 
 use Test::More;
+use Socket;
 
 use HTTP::Request          ();
 use HTTP::Response         ();
@@ -123,6 +124,7 @@ sub terminate {
     my ($self) = @_;
 
     kill 'KILL', $self->{'pid'};
+
     return $self->wait();
 }
 
@@ -140,7 +142,10 @@ sub _serve_socket {
             next;
         };
 
-        next if fork;
+        if (my $cpid = fork) {
+            diag "Forked handler PID $cpid";
+            next;
+        }
 
         my $ok = eval {
             my $we_are_done;
@@ -148,7 +153,7 @@ sub _serve_socket {
             while ( !$we_are_done ) {
                 my $hdr = do { local $/ = "\x0d\x0a\x0d\x0a"; <$peer> };
 
-                diag "Socket is done." if !$hdr;
+                diag "Handler PID $$ socket is done." if !$hdr;
                 last if !$hdr;
 
                 my $req = HTTP::Request->parse($hdr);
@@ -156,6 +161,7 @@ sub _serve_socket {
                 diag "Got request: " . $req->uri()->as_string();
 
                 if ( $req->uri()->as_string() =~ m<noanswer> ) {
+                    diag "Handler PID $$ self-terminating per request";
                     kill 'TERM', $$;
                 }
 
@@ -165,7 +171,7 @@ sub _serve_socket {
 
                 my $body;
 
-                if ($content_length) {
+                if (defined $content_length) {
                     diag "Expecting $content_length content bytes …";
 
                     $body = q<>;
@@ -181,7 +187,7 @@ sub _serve_socket {
                     }
                 }
                 else {
-                    diag "No Content-Length sent; assuming empty payload …";
+                    diag "No Content-Length sent in request; assuming empty payload …";
 
                     $body        = q<>;
                     $we_are_done = 1;
@@ -205,23 +211,32 @@ sub _serve_socket {
                     $resp_obj = $class->_get_response($req);
                 }
 
-                # We aren’t going to read any more, so stop listening.
-                shutdown $peer, 0;
-                1 while read $peer, my $buf, 65536;
-
                 $resp_obj->header( Connection     => 'close' );
                 $resp_obj->header( 'X-TestServer' => $class );
 
+                # Ideally we’d use unbuffered I/O, but since we already used
+                # buffered I/O to read we might as well use buffered to write.
                 print {$peer} 'HTTP/1.1 ' . $resp_obj->as_string("\x0d\x0a");
-            }
 
-            close $peer;
+                # This has to come after the write, or else TLS might get
+                # get confused. (OpenSSL’s write logic might try to read!)
+                #
+                shutdown $peer, Socket::SHUT_RD;
+                1 while read $peer, my $buf, 65536;
+
+                # NB: we can no longer manually close() the peer socket
+                # because it’ll fail its TLS shutdown.
+
+                diag "Handler PID $$ sent response";
+            }
 
             1;
         };
 
         warn "Server grandchild PID $$: $@" if !$ok;
 
+        diag "Handler PID $$ exit (ok? $ok)";
+
         exit( !$ok ? 1 : 0 );
     }
 
